TOOL_NAME = "Check Times Instrument Used"

--Renoise Keybinds and menus
-------------------------------
renoise.tool():add_keybinding {
  name = "Global:Tools:"..TOOL_NAME,
  invoke = function()main_toggle()end  
}

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Ledger`s Scripts:"..TOOL_NAME,
  invoke = function()main_start()end  
}

-----------------------
--intrument menu entry
-----------------------
renoise.tool():add_menu_entry {
  name = "Instrument Box:"..TOOL_NAME,
  invoke = function() main_start()
  end
}


--[[
------------------------------------------
--Set up Preferences file
------------------------------------------
--create xml
local options = renoise.Document.create {
  value_a = false,
  value_b = 10,
}
 --assign options-object to .preferences so renoise knows to load and update it with the tool
renoise.tool().preferences = options
------------------------------------------
------------------------------------------
--variable syntax 
--options.value_a.value
--]]

--------------------------------------------------------------------------------
--some basic colors for gui elements
--------------------------------------------------------------------------------

--[[
--e.g. For changing vb.views["sample present colour 2"].color when states change
COLOR_GREY = {0x30,0x42,0x42}
COLOR_ORANGE ={0xFF,0x66,0x00}
COLOR_YELLOW = {0xE0,0xE0,0x00}
COLOR_BLUE = {0x50,0x40,0xE0}  
COLOR_RED = {0xEE,0x10,0x10}
COLOR_GREEN = {0x20,0x99,0x20}
COLOR_RED_MILD = {0x90,0x10,0x10}

--Constants holding first index of color-code: Used to identify color in vb.views table i.e.
--if vb.views["button"].color[1] == COLOR_ORANGE_FLAG then --etc
COLOR_GREY_FLAG = COLOR_GREY[1]
COLOR_ORANGE_FLAG = COLOR_ORANGE[1]
COLOR_YELLOW_FLAG = COLOR_YELLOW[1]
COLOR_BLUE_FLAG = COLOR_BLUE[1]  
COLOR_RED_FLAG = COLOR_RED[1]
COLOR_GREEN_FLAG = COLOR_GREEN[1]
COLOR_RED_MILD_FLAG = COLOR_RED_MILD[1]


LUA_COUNTS_FROM_1 = 1
--]]

--global variables for gui
local my_dialog = nil 
local vb = nil


--toggle the tool open and closed (keyboard shortcut start-up)
-------------------------------------------------------------
function main_toggle()
----------------------
 --close dialog if it is open
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    --reset global my_dialog
     my_dialog = nil 
  else --run main
    main()
  end
end

--always open/ restart tool (menu entry start-up)
-------------------------------------------------
function main_start()
---------------------
  if (my_dialog and my_dialog.visible) then 
    --close dialog
    my_dialog:close()
    --reset global my_dialog
    my_dialog = nil 
  end
  --run main to re-initialise tool
  main()
end
----------------------------------------------------------



--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end
-------

--global to store string that displays all info. on the gui
readout_string = ""


--function scans the song and returns a formatted string with 2 sections:
--1) lists all tracks that contain notes with a target instrument_value
--2) lists all instances of notes in a single specific track (defaults to selected track in renoise)
--------------------------------------------------------
function update_readout_string(track_idx,instrument_idx)
--------------------------------------------------------
  --get values needed for looping
  local song = renoise.song()
  local instrument_num = instrument_idx or  song.selected_instrument_index
  local track_index = track_idx or song.selected_track_index
  local sub = string.sub
  --initialise global readout_string
  readout_string = ""
  
  --constants for looping line-strings to get the instrument number
  local FIRST_INST_NUM = 4
  local LAST_INST_NUM = 204
  local NEXT = 18

  
  --return with no modifications to readout_string  
  if song:track(track_index).type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
    return "\nWon`t Work On Non-Sequencer Tracks\n\nSelect A Sequencer Track And Press UPDATE"
  end
  --[[
  --local function scans a single track to find any instances of
  --notes with the instrument_value instrument_num --declared above
  ------------------------------------
  local function scan_track_for_note(trk)
  ------------------------------------
    local song = renoise.song()
    --return if track not a sequencer track
    if song:track(trk).type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
      return false
    end
    --loop patterns
    for pattern = 1,#song.patterns do
      local pattern_track = song:pattern(pattern):track(trk)
      --loop lines
      for line = 1,#pattern_track.lines do
        --loop columns
        for col = 1,12 do
          local note_col = pattern_track:line(line):note_column(col)
          if note_col.is_empty == false then
            --see if selected instrument value matches
            if note_col.instrument_value == instrument_num - 1 then
              return true
            end
          end--line
        end--cols
      end--line
    end--pattern
  end--fn
  -------------------------------------
  --]]
  
  local function scan_track_for_note(trk)
    --------------------------------------
   
    
    local song = renoise.song()
    --return if track not a sequencer track
    if song:track(trk).type ~= renoise.Track.TRACK_TYPE_SEQUENCER then
      return false
    end
    --loop patterns
    for pattern = 1,#song.patterns do
      local pattern_track = song:pattern(pattern):track(trk)
      --loop pattern track
      for i = 1,#pattern_track.lines do
        --get current line
        local line = pattern_track:line(i) 
        --weed out empty lines
        if (line.is_empty == false) then
          --convert current line to string
          local line_str = tostring(line)
          --loop through string characters of line, extracting the instrument numbers
          --and comparing them to the target instrument number
          for pos = FIRST_INST_NUM, LAST_INST_NUM, NEXT do
            if tonumber("0x" .. sub(line_str, pos, pos+1)) == instrument_num - 1 then
             return true
            end
          end
        end
      end 
    end
  end
    
  
  
  
  
  --using the above local function, popuplate table note_in_tracks  
  --with indexes of tracks that contain notes with target instrument_num
  local note_in_tracks = {}
  for track = 1,song.sequencer_track_count do
    if scan_track_for_note(track) == true then
      table.insert(note_in_tracks,track)
    end
  end
  
  --if no notes were found then return
  if #note_in_tracks == 0 then
    return "\nNO NOTES FOUND IN SONG"
  end
  
  --add in tracks where a note was found
  local tracks_with_notes_string = ""
  for i = 1, #note_in_tracks do
    tracks_with_notes_string = tracks_with_notes_string..tostring(note_in_tracks[i])..", "
  end
  --remove final comma as it is not needed after the last number
  --we do this by getting a sub string up to the last character of tracks_with_notes_string
  tracks_with_notes_string = string.sub(tracks_with_notes_string,1,(#tracks_with_notes_string-2)) --comma and space removed


  --start to build readout_string
  readout_string = readout_string.."\n".."HAS NOTES ON TRACK: "..tracks_with_notes_string.."\n".."-------------------------------------------------\n"

  --store readout string as we will reset it if no notes are found
  local stored_readout_string = readout_string.."\nNO NOTES FOUND IN CURRENTLY SELECTED `TRACK "..song.selected_track_index.."`: \n\n"
  
  readout_string = readout_string.."\nNOTES FOUND IN CURRENTLY SELECTED `TRACK "..song.selected_track_index.."`: \n\n"
  
  
 
  --[[
  --loop patterns to find all notes in selected (or target) track
  ---------------------------------------------------------------
  for pattern = 1,#song.patterns do

    local pattern_track = song:pattern(pattern):track(track_index)
    readout_string = readout_string.."Pattern: "..(pattern-1)..", Lines: "
   
    --loop lines
    for line = 1,#pattern_track.lines do
      --loop columns
      for col = 1,12 do
        local note_col = pattern_track:line(line):note_column(col)
        if note_col.is_empty == false then
          --see if selected instrument value matches
          if note_col.instrument_value == instrument_num - 1 then
            readout_string = readout_string..(line-1)..","
            note_found = true
          end
        end
      end
    end
    
    if note_found == true then
      readout_string = string.sub(readout_string,1,(#readout_string-1))
      note_found = false
    end
    --add newline to string after each pattern loop
    readout_string = readout_string.."\n"
  end--patterns loop (single track)
  --]]
  
  --flag to check if notes are found in the current track
  local notes_found_in_current_track = false
  --loop patterns (single track) to find all notes in selected (or target) track
  ---------------------------------------------------------------
  --flag to show at least one note is found in the pattern
  local note_found = false
  --loop all patterns
  for pattern = 1,#song.patterns do

    local pattern_track = song:pattern(pattern):track(track_index)
    readout_string = readout_string.."Pattern: "..(pattern-1)..", Lines: "
   
    --loop lines
    for i = 1,#pattern_track.lines do
      --get current line
      local line = pattern_track:line(i)
      --weed out empty lines
      if (line.is_empty == false) then 
        --convert current line to string
        local line_str = tostring(line)
        --loop through string characters of line, extracting the instrument numbers
        --and comparing them to the target instrument number
        for pos = FIRST_INST_NUM, LAST_INST_NUM, NEXT do
          if tonumber("0x" .. sub(line_str, pos, pos+1)) == instrument_num - 1 then
            readout_string = readout_string..(i-1)..","
            note_found = true
            notes_found_in_current_track = true
            break --so we don`t get the line referenced multiple times, i.e. when chords are present
          end
        end
      end
    end--lines in pattern

    --if a note was found then remove last comma from end of readout string
    if note_found == true then
      readout_string = string.sub(readout_string,1,(#readout_string-1))
      note_found = false
    end
    --add newline to string after each pattern loop
    readout_string = readout_string.."\n"
  end--all patterns loop (single track)
  
  --if no notes were found then go back to stored_readout_string which says:
  --"NO NOTES FOUND IN CURRENTLY SELECTED `TRACK`"
  if notes_found_in_current_track == false then
    readout_string = stored_readout_string
  end
  --return the completed readout_string
  return readout_string
end

------------------------------------
------------------------------------
function main()
------------------------------------
------------------------------------
 

  --toggle dialog so if its already open then close it.  The timer will catch
  --that it is closed and do the housekeeping like removing notifiers (and timer itself)
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    return
  end
  
  --renoise song object
  local song = renoise.song()
  
  --helper, convert dec to a hex string
  ----------------------------------
  local function number_to_hex_string(num)
  ----------------------------------
    local string = string.upper(string.format("%x", num))
    --prefix numbers lower than 15 with a zero (0F will be 16 counting from 0)
    if num < 16 then
      string = "0"..string
    end
    return string
  end
   
  --get instrument names
  -------------------------------------
  local function get_instrument_names()
  --------------------------------------
    local inst_tab = {}
    for i = 1,#song.instruments do
      --prefix instrument names wit the instrument number
      inst_tab[i] = number_to_hex_string(i-1)..": "..song:instrument(i).name

    end
    return inst_tab
  end
  
  --get track names
  --------------------------------
  local function get_track_names()
  --------------------------------
    local track_tab = {}
    for i = 1,#song.instruments do
      track_tab[i] = i..": "..song:track(i).name
    end
    return track_tab
  end

  --set globals
  -------------
  --assign global vb to viewbuilder
  vb = renoise.ViewBuilder()
  
  ----------------------------------------------
  --GUI
  ----------------------------------------------     

  --variables that will be added to
  --dialog_content:add_child(send_row)
  local my_first_row = vb:column{}
  -------------------------------
  local my_first_element = nil
  

    my_first_element = vb:column{
                        id = "main col",
                        vb:row{
                          margin = 4,
                          vb:popup{
                           width = 250,
                           items = get_instrument_names(),
                           value = song.selected_instrument_index,
                           id = "inst. select",
                           notifier = function(value)
                                     --update text
                                     vb.views["readout string"].text = "\nClick UPDATE"
                                     --vb.views["readout string"].text = update_readout_string(nil,value)
                                    end
                          }
                         },
                       
                         vb:row{
                          margin = 4,
                            vb:button{
                              text = "UPDATE",
                              notifier = function()
                                           --get inst. popup value
                                           local selected_inst_popup = vb.views["inst. select"].value
                                           --get string from function
                                           vb.views["readout string"].text = update_readout_string(nil,selected_inst_popup)
                                         end
                        
                          }--button
                         },--row

                        vb:multiline_textfield { 
                         id = "readout string",
                         width = 310,
                         height = 200,
                         text = update_readout_string()
                        }
                       }


    my_first_row:add_child(my_first_element) 
  

  
  --------------------------------------------------------
  --------------------------------------------------------
  --dialog content will contain all of gui; passed to renoise.app():show_custom_dialog()
  local dialog_content = vb:column{}
  dialog_content:add_child(my_first_row)
 
  --------------
  --key Handler
  --------------
  local function my_keyhandler_func(dialog,key)
     --toggle lock focus hack, allows pattern ed to get key input
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     return key
  end 
  
 
  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      TOOL_NAME, dialog_content,my_keyhandler_func)

 --[[ 
  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(timer) then
    renoise.tool():add_timer(timer,80)
  end
  --]]
  
  --------------------------------------------    
  --close dialog function ON NEW SONG
  --------------------------------------------
  local function closer(d)
    --close dialog if exists and is open     
    if (d ~= nil) and (d.visible == true) then
      d:close()
     -- remove_notifiers()
    end
    --reset global my_dialog
     my_dialog = nil 
  end
  -- notifier to close dialog on load new song
  renoise.tool().app_release_document_observable:add_notifier(closer,my_dialog)
  -------------------------------------------------------------------------------
  
  --first run of timer
  -- timer()
end

--[[
----------------
--timer function
----------------
function timer()
  
  --do stuff
  
  --remove timer when GUI is closed
  if(my_dialog == nil) or (my_dialog.visible == false) then
    if renoise.tool():has_timer(timer) then
      renoise.tool():remove_timer(timer)
    end
  end
end
--]]



